%% Author- Dhanush R,
% PhD Scholar (AM13D021)
% Indian Institute of Technology, Madras
% India
% Code is used to determine the Displacement Ratio and Smoothness metric 
% Code to used to interpolate the representative data from a single subject
clear all;
close all;
clc;
% 10 subjects participated in the experiment 5 males and 5 females 1 male(
% subject7) and 1 female (subject8) data was not used for analysis
for sub=1:10
if sub == 7 || sub == 8
   continue;
else
% i specifies the total number of tasks in the experiment, j specifies
% the total number of trials in a task.
for i=1:12
   for j=1:15 
       count=1;
% input raw data in to the code.
    dname=['D:\Data\TTP\data1\Subject', num2str(sub),  '\','Subject',num2str(sub),'_','Task',num2str(i),'_Trial',num2str(j),'.csv'];   
csvdata=csvread(dname);

% Segregate the dispalcement data from the raw data based on the task
% specified in the above for loop
    
    disp_phalange=csvdata(:,6*(i-1)+1:6*(i-1)+3);
    ref_wrist=csvdata(:,91:93);
    disp_first_thumb=csvdata(:,73:75);
    %input parameters for spectral arc length method, see the function
    %SpectralArcLength.m for more details
    param= [0.05, 5, 4];   
%Calculate measurements with respect to wrist frame of reference.
    corr_disp_phalange=ref_wrist-disp_phalange;
    corr_disp_thumb=ref_wrist-disp_first_thumb;

     % Convert the units from inches to cm wherever required 1 inch is
    % 2.54cm
      % Filter the kinematic data at 5 hz   
           if sub == 1 || sub == 2 
     filt_p1= BW_Filter(corr_disp_phalange*2.54,100,5);
         filt_t1= BW_Filter(corr_disp_thumb*2.54,100,5);
      else
         filt_p1= BW_Filter(corr_disp_phalange,100,5);
         filt_t1= BW_Filter(corr_disp_thumb,100,5);   
      end
            % Determine the  resultant velocity and displacement from 3 axis
   % kinematic data for both phalanx and thumb data seperately using
   % fivepoint stencil

           vel_p1=sqrt((fivepointderivative(filt_p1(:,1),0.01)).^2+(fivepointderivative(filt_p1(:,2),0.01)).^2+(fivepointderivative(filt_p1(:,3),0.01)).^2) ;
           vel_t1=sqrt((fivepointderivative(filt_t1(:,1),0.01)).^2+(fivepointderivative(filt_t1(:,2),0.01)).^2+(fivepointderivative(filt_t1(:,3),0.01)).^2) ;
           disp_p1=sqrt((filt_p1(:,1)).^2+(filt_p1(:,2)).^2+(filt_p1(:,3)).^2) ;
           disp_t1=sqrt((filt_t1(:,1)).^2+(filt_t1(:,2)).^2+(filt_t1(:,3)).^2) ;

%Detemine the movement onset and end of movement for thumb and finger
%phalanx seperately
 seg_det=seg_mov(disp_p1);
seg_det1=seg_mov(disp_t1);

          
%Segregate the dispalcement and velocity of thumb and finger phalanx
  
dp{i,j}=disp_p1(seg_det{2}:seg_det{1});
dt{i,j}=disp_t1(seg_det1{2}:seg_det1{1});

vp{i,j}=vel_p1(seg_det{2}:seg_det{1});
vt{i,j}=vel_t1(seg_det1{2}:seg_det1{1});

%Detemine the smoothness using spectral arc method as discussed by
%balasubramanian et al.,(2012)
sp(i,j)=SpectralArcLength(vel_p1(seg_det{2}:seg_det{1}),0.01,param);
st(i,j)=SpectralArcLength(vel_t1(seg_det1{2}:seg_det1{1}),0.01,param);

   end
 
    
end
    
     
end
% Store the segemented data for determining displacement ratio
 dp_sub{sub}=dp;
 dt_sub{sub}=dt;
  vp_sub{sub}=vp;
 vt_sub{sub}=vt;
%Determine average across trials
   
    sp_sub1(sub,:)=mean(sp');
   st_sub1(sub,:)=mean(st');
end


disp_p_sub1=dp_sub( ~cellfun('isempty',dp_sub));
disp_t_sub1=dt_sub( ~cellfun('isempty',dt_sub));

% Displacement ratio (z-transformed value) for thumb and finger phalanges determined seperately 
for sub=1:8
   for i=1:12     
       for j=1:15
    
 y_p=max(disp_p_sub1{1,sub}{i,j})-min(disp_p_sub1{1,sub}{i,j});
 y_t=max(disp_t_sub1{1,sub}{i,j})-min(disp_t_sub1{1,sub}{i,j});;
 tot =y_p+y_t;
 d_m_t(i,j)=y_t/tot;
 temp_dmt=y_t/tot;
 d_m_p(i,j)=y_p/tot;
 temp_dmp=y_p/tot;
 zt(i,j)=.5*log((1+temp_dmt)./(1-temp_dmt));
zp(i,j)=.5*log((1+temp_dmp)./(1-temp_dmp));

       end
     
   end  
%    figure(sub);
%    histogram(d_m_p);
   d_m_t_sub(sub,:)=mean(d_m_t');
   zt_sub(sub,:)=mean(zt');
   d_m_p_sub(sub,:)=mean(d_m_p');
   zp_sub(sub,:)=mean(zp');
end


stack_disp_pz_mean=mean(zp_sub);
stack_disp_tz_mean=mean(zt_sub);

stack_disp_p_mean=mean(d_m_p_sub);
stack_disp_t_mean=mean(d_m_t_sub);

%% Plot representative data from a single subject. 
%segregated (with help of movement onset and end of movement)
%data from a single subject was interpolated to 100 datapoints
%Interpolation was performed using cubic spline interpolation
%Interpolated data in variables inter_dp, inter_dt etc was exported 
%to origin for producing plots.
r_dp =load('r_dp');
r_dt =load('r_dt');
r_vp =load('r_vp');
r_vt =load('r_vt');
dp=r_dp.dp;
dt=r_dt.dt;
vp=r_vp.vp;
vt=r_vt.vt;
     
 % interpolation of displacement profile data for thumb and finger phalanx
      for j=1:12
           ind_p=size((dp{j}),1);
ind_t=size((dt{j}),1);
           x_p=1:ind_p;
           x_t=1:ind_t;
%            

 y_p=dp{j}-mean(dp{j}(1:5));
 y_t=dt{j}-mean(dt{j}(1:5));


int_ind_p=linspace(1,ind_p,100);
int_ind_t=linspace(1,ind_t,100);
temp_p(j,:)=interp1(x_p,y_p, int_ind_p ,'spline');
temp_t(j,:)=interp1(x_t,y_t, int_ind_t ,'spline');

       end
     
 % interpolation of velocity profile data for thumb and finger phalanx
       for j=1:12
           ind_p1=size((vp{j}),1);
ind_t1=size((vt{j}),1);
           x_p1=1:ind_p1;
           x_t1=1:ind_t1;

 y_p1=vp{j};
 y_t1=vt{j};


int_ind_p1=linspace(1,ind_p1,100);
int_ind_t1=linspace(1,ind_t1,100);
temp_p_v(j,:)=interp1(x_p1,y_p1, int_ind_p1 ,'spline');
temp_t_v(j,:)=interp1(x_t1,y_t1, int_ind_t1 ,'spline');

       end

       
inter_dp=dp1;
inter_dt =dt1;
inter_vp =vp1;
inter_vt =vt1;

