function segout = seg_mov(data)
%% 
% The following function is used to determine the onward and end of
% movement
% This uses the following Algorithm
% Step1:  Maximum value in the velocity profile will be determined
% Step2: Left side of the velocity profile was traversed from the maximum value, until the value becomes less than 5 % of the maximum velocity. The successive value is taken as the onset of movement.
% % for example, if maximum value in the velocity profile is found at 61 then traversing left from 61, if 47th value is less than 5% of max value. Then the onset of movement was taken from 48th value.
% % Step3: Right from the maximum value was traversed in the velocity profile, until the value becomes less than 5 % of the maximum velocity. The predecessor value is taken as the end of movement. 
% % Similarly,  if 88th value is less than 5% of max value (61). Then the end of movement is taken till 87th value.
% Function also overcomes glitches in the displacement profile, by
% calculating lower and upper bound expected displacement values and
% comapares with the actual onset and end of movement to avoid false
% movement onset and end of movement times.
% % Function usage :
% [end of movement, onset of movement]=seg(input displacement profile)
%% 
%input data is initialized and velocity and accelerations are determined
count=1;
count1=1;
temp_sign_st=[]; 
temp_sign_ed=[]; 
temp_dis_p=data(1:300);
[temp_vel,tem_acc]= fivepointderivative(temp_dis_p,0.01);     
         
%find the min of the velocity profile to segregate onward movement

         eom_p=find(temp_vel==min(temp_vel));
         eom_p1=find(temp_dis_p==max(temp_dis_p));
         
%To remove un wanted noise in the start of the trial this is 
%checked in both velocity and displacment profiles these  sample point values 
%of 10;20;75  are determined after manually checking the velocity and
%displacement profiles
         if eom_p1==1
           temp_dis_p(1:10)=[];
            eom_p1=find(temp_dis_p==min(temp_dis_p));
                                                                                                               
         end
         
         if eom_p<20
           temp_vel(1:20)=[];
           eom_p=find(temp_vel==min(temp_vel));
                                                                                                               
         end
 %offset values are calculated to account for deleted noise 
 %(unwanted jerks in the signal at the start of the trial) in the data
         if eom_p1<=75 
             a_som_p=1;
             a_eom_p=eom_p1;  
                  else
                  temp_vel= temp_vel(1:eom_p);
                 tem_acc=tem_acc(1:eom_p);
                 ind_p=find(abs(tem_acc(1:20))>170);
                 if isempty(ind_p)==1
                          temp_vel=temp_vel;
                          off_val_p=0;
                 else
                 off_val_p=ind_p(end);
                 temp_vel(ind_p(1):ind_p(end)+5)=[];
                 end
                 
     %Determine the maximum velcoity (positive peak) from the velocity profile            
             temp_vel_max=find(temp_vel(1:end)==max(temp_vel(1:end))); 
      %Detemine the expected values of displacements, lower bound and upper
      %bound values
             % detemine the lower bound of displacement from the
             % velocity profile
             %lower bound values are calcualted by finding the velocity
             %values between 0.5 cm/sec and -0.5cm/sec and the
             %corresponding values in the displacement profile was averaged
             %to obtain the lower bound value
             lwbound_disp=mean(temp_dis_p(find(temp_vel(1:temp_vel_max)<0.5&temp_vel(1:temp_vel_max)>-0.5)));
      lwbound_disp(isnan(lwbound_disp))=1;
             
%Segregate the velcoity profile in to two parts on from its maximum value
%towards the left from the start of trial and another from maximum towards
%right until its negative peak.

           
             temp_vel_f=flip(temp_vel(1:temp_vel_max));
             tem_acc_f=  tem_acc(1:temp_vel_max);
             temp_vel_b=temp_vel(temp_vel_max:end);
             tem_acc_b=  tem_acc(temp_vel_max:end);
             tem_vel_b_f=  flip(temp_vel(temp_vel_max:end));
              
                temp_start=find(temp_vel_f<0.05*max(temp_vel_f));
               
                for k=1:length(tem_acc_f)-1
                              if sign(tem_acc_f(k)*sign(tem_acc_f(k+1))) == -1
                                 temp_sign_st(k)=k;  
                                                               
                                 % Or whatever you want to do with the idx
                             end 
            
                    
                end
                  %Find the start of movement 
             if isempty(temp_start)==1
                 if isempty(temp_sign_st)==1
                 temp_start1=1;
                 else
                   temp_start1=(temp_vel_max-(temp_sign_st(1)-1))+1;  
                 end
             else
                 temp_start1=(temp_vel_max-(temp_start(1)-1))+1;
                 
             end
             
               %Compare the start of movement with the lower bound value, 
               %if the condition of being closer to the expected value of
               %displacement was not met then the loop is iterated until the
               %condition is met
                 while abs((temp_dis_p(temp_start1)-lwbound_disp))>0.1
                     
                          count=count+1;

if isempty(temp_start)==0
                temp_start1=(temp_vel_max-(temp_start(count)-1))+1;
else
    break;
                
end
if count>=length(temp_start)
                        break ;
                     end
                 end
         
         
                 %Find the end of movement
         temp_end=find(temp_vel_b<0.05*max(temp_vel_b));
          for k=1:length(tem_vel_b_f)-1
                              if sign(tem_vel_b_f(k)*sign(tem_vel_b_f(k+1))) == -1
                                 temp_sign_ed(k)=k+1;    % Or whatever you want to do with the idx
                             end 
            
                    
          end
           %Compare the end of movement with the upper bound, find the
           %first sign change in the velocity profile between the negative
           %peak and the positive peak. The corresponding value in the
           %displacement profile will the upper bound value.
            if isempty(temp_sign_ed)==1
              upbound_disp=max(data);
            else
                 up_temp= find(temp_sign_ed);
             upbound_disp=temp_dis_p(temp_vel_max+(size(temp_vel_b,1)-up_temp(1)+1));
          
            end
             if isempty(temp_end)==1
                  if isempty(temp_sign_ed)==1
                 temp_end1=1;
                 else
                   temp_end1=(temp_vel_max+(temp_sign_ed(1)-1))+1;
                  end
             else
        temp_end1=(temp_vel_max+(temp_end(1)-1))+1;
                 
             end
             %  Compare the end of movement with the upper bound value, if the
               %condition of being closer to the expected value of
               %displacement was not met then the loop will be iterated until the
               %condition is met
              while abs((upbound_disp-temp_dis_p(temp_end1)))>0.1
                   

                   count1=count1+1;
             if isempty(temp_end)==0
temp_end1=(temp_vel_max+(temp_end(count1)-1));
              else
                  break;
             end
               if count1>=length(temp_end)
                        break ;
                     end
             
               end
               
               %Determine the actual movement start and end.
             if isempty(temp_start)==1
              som_p=1;
             else
%               temp_start=temp_start;
              som_p= temp_start1;
             end
             
                if isempty(temp_end)==1
              t_eom_p=eom_p;
             else
%               temp_end=temp_end;
              t_eom_p= temp_end1;
             end
                       
                        
               if isempty(temp_vel_max)==1||isempty(temp_start)==1||eom_p<5
               som_p=1;
              t_eom_p=eom_p;
               
               end 
               a_eom_p=t_eom_p+off_val_p;
         a_som_p=som_p+off_val_p;
          
               
            
         end
         
         
       segout{1}=a_eom_p;

 segout{2}=a_som_p;
end